package ru.ulstu.method.fuzzy;

import org.springframework.stereotype.Component;
import ru.ulstu.datamodel.Model;
import ru.ulstu.datamodel.exception.ModelingException;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.datamodel.ts.TimeSeriesValue;
import ru.ulstu.method.Method;
import ru.ulstu.method.MethodParamValue;
import ru.ulstu.method.MethodParameter;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

@Component
public class PlainFuzzy extends Method {
    @Override
    public List<MethodParameter> getAvailableParameters() {
        return PlainFuzzyModel.getAvailableParameters();
    }

    @Override
    protected Model getModelOfValidTimeSeries(TimeSeries timeSeries, List<MethodParamValue> parameters) {
        PlainFuzzyModel model = new PlainFuzzyModel(timeSeries, parameters);
        List<Triangle> fuzzySets = generateFuzzySets(timeSeries, model.getNumberOfFuzzyTerms().getIntValue());
        return null;
    }

    private List<Triangle> generateFuzzySets(TimeSeries timeSeries, Integer numberOfFuzzyTerms) {
        List<Triangle> fuzzySets = new ArrayList<Triangle>();
        double min = timeSeries.getValues()
                .stream()
                .min(Comparator.comparing(TimeSeriesValue::getValue))
                .map(TimeSeriesValue::getValue)
                .orElseThrow(() -> new RuntimeException("Минимальное значение не найдено"));
        double max = timeSeries.getValues()
                .stream()
                .max(Comparator.comparing(TimeSeriesValue::getValue))
                .map(TimeSeriesValue::getValue)
                .orElseThrow(() -> new RuntimeException("Максимальное значение не найдено"));
        double delta = ((max - min) / numberOfFuzzyTerms) * 2;
        for (int i = 0; i < numberOfFuzzyTerms; i++) {
            fuzzySets.add(new Triangle(min + i * delta - delta,
                    min + i * delta,
                    min + i * delta + delta));
        }
        return fuzzySets;
    }

    @Override
    protected TimeSeries getForecastWithValidParams(Model model, TimeSeries forecast) throws ModelingException {
        return null;
    }

    @Override
    public String getName() {
        return "Нечеткая модель";
    }
}
