package ru.ulstu.method.fuzzy;

import ru.ulstu.datamodel.Model;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.method.MethodParamValue;
import ru.ulstu.method.MethodParameter;
import ru.ulstu.method.fuzzy.parameter.NumberOfFuzzyTerms;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class PlainFuzzyModel extends Model {
    private final static String PREFIX_OF_FUZZY_LABEL = "X";
    private final MethodParamValue numberOfFuzzyTerms = new MethodParamValue(NumberOfFuzzyTerms.getInstance(), 2);
    private List<String> fuzzyTimeSeries = new ArrayList<>();
    private TimeSeries defuzzyfiedTimeSeries;

    protected PlainFuzzyModel(TimeSeries ts, List<MethodParamValue> parameters) {
        super(ts);
        defuzzyfiedTimeSeries = new TimeSeries("Defuzzified time series of ", ts.getKey());
        for (MethodParamValue parameter : parameters) {
            if (parameter.getParameter() instanceof NumberOfFuzzyTerms) {
                numberOfFuzzyTerms.setValue(parameter.getValue());
            }
        }
    }

    public static List<MethodParameter> getAvailableParameters() {
        return Collections.singletonList(NumberOfFuzzyTerms.getInstance());
    }

    public MethodParamValue getNumberOfFuzzyTerms() {
        return numberOfFuzzyTerms;
    }

    public List<String> getFuzzyTimeSeries() {
        return fuzzyTimeSeries;
    }

    public void setFuzzyTimeSeries(List<String> fuzzyTimeSeries) {
        this.fuzzyTimeSeries = fuzzyTimeSeries;
    }

    public TimeSeries getDefuzzyfiedTimeSeries() {
        return defuzzyfiedTimeSeries;
    }

    public void setDefuzzifiedTimeSeries(TimeSeries defuzzifiedTimeSeries) {
        this.defuzzyfiedTimeSeries = defuzzifiedTimeSeries;
    }
}
